@@ -12,8 +12,6 @@ TODO:
 	http://trac.flipturn.org/browser/trunk/peppy/lib/column_autosize.py
 """
 #================================================================
-# $Source: /home/ncq/Projekte/cvs2git/vcs-mirror/gnumed/gnumed/client/wxpython/gmListWidgets.py,v $
-# $Id: gmListWidgets.py,v 1.37 2010-02-06 21:37:26 ncq Exp $
 __version__ = "$Revision: 1.37 $"
 __author__ = "Karsten Hilbert <Karsten.Hilbert@gmx.net>"
 __license__ = "GPL"
@@ -28,12 +26,13 @@ import wx.lib.mixins.listctrl as listmixins
 
 if __name__ == '__main__':
 	sys.path.insert(0, '../../')
-from Gnumed.business import gmPerson
 from Gnumed.pycommon import gmTools, gmDispatcher
 from Gnumed.wxpython import gmGuiHelpers
 from Gnumed.wxGladeWidgets import wxgGenericListSelectorDlg, wxgGenericListManagerPnl
 
 #================================================================
+# FIXME: configurable callback on double-click action
+
 def get_choices_from_list (
 			parent=None,
 			msg=None,
@@ -51,16 +50,29 @@ def get_choices_from_list (
 			ignore_OK_button=False,
 			left_extra_button=None,
 			middle_extra_button=None,
-			right_extra_button=None):
+			right_extra_button=None,
+			list_tooltip_callback=None):
 	"""Let user select item(s) from a list.
 
-	- edit_callback: (item data)
 	- new_callback: ()
+	- edit_callback: (item data)
 	- delete_callback: (item data)
 	- refresh_callback: (listctrl)
+	- list_tooltip_callback: (item data)
 
-	returns None if cancelled
-	returns list (may be empty) of selected items
+	- left/middle/right_extra_button: (label, tooltip, <callback>)
+		<callback> is called with item_data as the only argument
+
+	returns:
+		on [CANCEL]: None
+		on [OK]:
+			if any items selected:
+				list of selected items
+			else:
+				if can_return_empty is True:
+					empty list
+				else:
+					None
 	"""
 	if caption is None:
 		caption = _('generic multi choice dialog')
@@ -74,6 +86,7 @@ def get_choices_from_list (
 	dlg.edit_callback = edit_callback
 	dlg.new_callback = new_callback
 	dlg.delete_callback = delete_callback
+	dlg.list_tooltip_callback = list_tooltip_callback
 
 	dlg.ignore_OK_button = ignore_OK_button
 	dlg.left_extra_button = left_extra_button
@@ -87,7 +100,7 @@ def get_choices_from_list (
 		dlg.set_column_widths()
 
 	if data is not None:
-		dlg.set_data(data=data)						# can override data set if refresh_callback is not None
+		dlg.set_data(data = data)					# can override data set if refresh_callback is not None
 
 	if selections is not None:
 		dlg.set_selections(selections = selections)
@@ -107,6 +120,8 @@ def get_choices_from_list (
 class cGenericListSelectorDlg(wxgGenericListSelectorDlg.wxgGenericListSelectorDlg):
 	"""A dialog holding a list and a few buttons to act on the items."""
 
+	# FIXME: configurable callback on double-click action
+
 	def __init__(self, *args, **kwargs):
 
 		try:
@@ -116,10 +131,7 @@ class cGenericListSelectorDlg(wxgGenericListSelectorDlg.wxgGenericListSelectorDl
 
 		wxgGenericListSelectorDlg.wxgGenericListSelectorDlg.__init__(self, *args, **kwargs)
 
-		if msg is None:
-			self._LBL_message.Hide()
-		else:
-			self._LBL_message.SetLabel(msg)
+		self.message = msg
 
 		self.left_extra_button = None
 		self.middle_extra_button = None
@@ -180,7 +192,11 @@ class cGenericListSelectorDlg(wxgGenericListSelectorDlg.wxgGenericListSelectorDl
 			return
 		if self.refresh_callback is None:
 			return
-		self.refresh_callback(lctrl = self._LCTRL_items)
+		wx.BeginBusyCursor()
+		try:
+			self.refresh_callback(lctrl = self._LCTRL_items)
+		finally:
+			wx.EndBusyCursor()
 		self._LCTRL_items.set_column_widths()
 	#------------------------------------------------------------
 	def _on_edit_button_pressed(self, event):
@@ -190,7 +206,11 @@ class cGenericListSelectorDlg(wxgGenericListSelectorDlg.wxgGenericListSelectorDl
 			return
 		if self.refresh_callback is None:
 			return
-		self.refresh_callback(lctrl = self._LCTRL_items)
+		wx.BeginBusyCursor()
+		try:
+			self.refresh_callback(lctrl = self._LCTRL_items)
+		finally:
+			wx.EndBusyCursor()
 		self._LCTRL_items.set_column_widths()
 	#------------------------------------------------------------
 	def _on_delete_button_pressed(self, event):
@@ -203,7 +223,11 @@ class cGenericListSelectorDlg(wxgGenericListSelectorDlg.wxgGenericListSelectorDl
 			return
 		if self.refresh_callback is None:
 			return
-		self.refresh_callback(lctrl = self._LCTRL_items)
+		wx.BeginBusyCursor()
+		try:
+			self.refresh_callback(lctrl = self._LCTRL_items)
+		finally:
+			wx.EndBusyCursor()
 		self._LCTRL_items.set_column_widths()
 	#------------------------------------------------------------
 	def _on_left_extra_button_pressed(self, event):
@@ -212,7 +236,11 @@ class cGenericListSelectorDlg(wxgGenericListSelectorDlg.wxgGenericListSelectorDl
 			return
 		if self.refresh_callback is None:
 			return
-		self.refresh_callback(lctrl = self._LCTRL_items)
+		wx.BeginBusyCursor()
+		try:
+			self.refresh_callback(lctrl = self._LCTRL_items)
+		finally:
+			wx.EndBusyCursor()
 		self._LCTRL_items.set_column_widths()
 	#------------------------------------------------------------
 	def _on_middle_extra_button_pressed(self, event):
@@ -221,7 +249,11 @@ class cGenericListSelectorDlg(wxgGenericListSelectorDlg.wxgGenericListSelectorDl
 			return
 		if self.refresh_callback is None:
 			return
-		self.refresh_callback(lctrl = self._LCTRL_items)
+		wx.BeginBusyCursor()
+		try:
+			self.refresh_callback(lctrl = self._LCTRL_items)
+		finally:
+			wx.EndBusyCursor()
 		self._LCTRL_items.set_column_widths()
 	#------------------------------------------------------------
 	def _on_right_extra_button_pressed(self, event):
@@ -230,7 +262,11 @@ class cGenericListSelectorDlg(wxgGenericListSelectorDlg.wxgGenericListSelectorDl
 			return
 		if self.refresh_callback is None:
 			return
-		self.refresh_callback(lctrl = self._LCTRL_items)
+		wx.BeginBusyCursor()
+		try:
+			self.refresh_callback(lctrl = self._LCTRL_items)
+		finally:
+			wx.EndBusyCursor()
 		self._LCTRL_items.set_column_widths()
 	#------------------------------------------------------------
 	# properties
@@ -250,6 +286,7 @@ class cGenericListSelectorDlg(wxgGenericListSelectorDlg.wxgGenericListSelectorDl
 		if definition is None:
 			self._BTN_extra_left.Enable(False)
 			self._BTN_extra_left.Hide()
+			self.__left_extra_button_callback = None
 			return
 
 		(label, tooltip, callback) = definition
@@ -267,6 +304,7 @@ class cGenericListSelectorDlg(wxgGenericListSelectorDlg.wxgGenericListSelectorDl
 		if definition is None:
 			self._BTN_extra_middle.Enable(False)
 			self._BTN_extra_middle.Hide()
+			self.__middle_extra_button_callback = None
 			return
 
 		(label, tooltip, callback) = definition
@@ -284,6 +322,7 @@ class cGenericListSelectorDlg(wxgGenericListSelectorDlg.wxgGenericListSelectorDl
 		if definition is None:
 			self._BTN_extra_right.Enable(False)
 			self._BTN_extra_right.Hide()
+			self.__right_extra_button_callback = None
 			return
 
 		(label, tooltip, callback) = definition
@@ -322,8 +361,6 @@ class cGenericListSelectorDlg(wxgGenericListSelectorDlg.wxgGenericListSelectorDl
 
 	def _set_edit_callback(self, callback):
 		if callback is not None:
-			if self.refresh_callback is None:
-				raise ValueError('refresh callback must be set before edit callback can be set')
 			if not callable(callback):
 				raise ValueError('<edit> callback is not a callable: %s' % callback)
 		self.__edit_callback = callback
@@ -361,7 +398,11 @@ class cGenericListSelectorDlg(wxgGenericListSelectorDlg.wxgGenericListSelectorDl
 		return self.__refresh_callback
 
 	def _set_refresh_callback_helper(self):
-		self.refresh_callback(lctrl = self._LCTRL_items)
+		wx.BeginBusyCursor()
+		try:
+			self.refresh_callback(lctrl = self._LCTRL_items)
+		finally:
+			wx.EndBusyCursor()
 		self._LCTRL_items.set_column_widths()
 
 	def _set_refresh_callback(self, callback):
@@ -373,6 +414,22 @@ class cGenericListSelectorDlg(wxgGenericListSelectorDlg.wxgGenericListSelectorDl
 			wx.CallAfter(self._set_refresh_callback_helper)
 
 	refresh_callback = property(_get_refresh_callback, _set_refresh_callback)
+	#------------------------------------------------------------
+	def _set_list_tooltip_callback(self, callback):
+		self._LCTRL_items.item_tooltip_callback = callback
+
+	list_tooltip_callback = property(lambda x:x, _set_list_tooltip_callback)
+	#def _get_tooltip(self, item):		# inside class
+	#def _get_tooltip(item):			# outside class
+	#------------------------------------------------------------
+	def _set_message(self, message):
+		if message is None:
+			self._LBL_message.Hide()
+			return
+		self._LBL_message.SetLabel(message)
+		self._LBL_message.Show()
+
+	message = property(lambda x:x, _set_message)
 #================================================================
 class cGenericListManagerPnl(wxgGenericListManagerPnl.wxgGenericListManagerPnl):
 	"""A panel holding a generic multi-column list and action buttions."""
@@ -439,7 +496,11 @@ class cGenericListManagerPnl(wxgGenericListManagerPnl.wxgGenericListManagerPnl):
 			return
 		if self.refresh_callback is None:
 			return
-		self.refresh_callback(lctrl = self._LCTRL_items)
+		wx.BeginBusyCursor()
+		try:
+			self.refresh_callback(lctrl = self._LCTRL_items)
+		finally:
+			wx.EndBusyCursor()
 	#------------------------------------------------------------
 	def _on_edit_button_pressed(self, event):
 		item = self._LCTRL_items.get_selected_item_data(only_one=True)
@@ -449,7 +510,11 @@ class cGenericListManagerPnl(wxgGenericListManagerPnl.wxgGenericListManagerPnl):
 			return
 		if self.refresh_callback is None:
 			return
-		self.refresh_callback(lctrl = self._LCTRL_items)
+		wx.BeginBusyCursor()
+		try:
+			self.refresh_callback(lctrl = self._LCTRL_items)
+		finally:
+			wx.EndBusyCursor()
 	#------------------------------------------------------------
 	def _on_remove_button_pressed(self, event):
 		item = self._LCTRL_items.get_selected_item_data(only_one=True)
@@ -459,7 +524,11 @@ class cGenericListManagerPnl(wxgGenericListManagerPnl.wxgGenericListManagerPnl):
 			return
 		if self.refresh_callback is None:
 			return
-		self.refresh_callback(lctrl = self._LCTRL_items)
+		wx.BeginBusyCursor()
+		try:
+			self.refresh_callback(lctrl = self._LCTRL_items)
+		finally:
+			wx.EndBusyCursor()
 	#------------------------------------------------------------
 	# properties
 	#------------------------------------------------------------
@@ -472,8 +541,129 @@ class cGenericListManagerPnl(wxgGenericListManagerPnl.wxgGenericListManagerPnl):
 
 	new_callback = property(_get_new_callback, _set_new_callback)
 #================================================================
+from Gnumed.wxGladeWidgets import wxgItemPickerDlg
+
+class cItemPickerDlg(wxgItemPickerDlg.wxgItemPickerDlg):
+
+	def __init__(self, *args, **kwargs):
+
+		try:
+			msg = kwargs['msg']
+			del kwargs['msg']
+		except KeyError:
+			msg = None
+
+		wxgItemPickerDlg.wxgItemPickerDlg.__init__(self, *args, **kwargs)
+
+		if msg is None:
+			self._LBL_msg.Hide()
+		else:
+			self._LBL_msg.SetLabel(msg)
+
+		self._LCTRL_left.activate_callback = self.__pick_selected
+		#self._LCTRL_left.item_tooltip_callback = self.__on_get_item_tooltip
+
+		self._LCTRL_left.SetFocus()
+	#------------------------------------------------------------
+	# external API
+	#------------------------------------------------------------
+	def set_columns(self, columns=None, columns_right=None):
+		self._LCTRL_left.set_columns(columns = columns)
+		if columns_right is None:
+			self._LCTRL_right.set_columns(columns = columns)
+		else:
+			if len(columns_right) < len(columns):
+				cols = columns
+			else:
+				cols = columns_right[:len(columns)]
+			self._LCTRL_right.set_columns(columns = cols)
+	#------------------------------------------------------------
+	def set_string_items(self, items = None):
+		self._LCTRL_left.set_string_items(items = items)
+		self._LCTRL_left.set_column_widths()
+		self._LCTRL_right.set_string_items()
+
+		self._BTN_left2right.Enable(False)
+		self._BTN_right2left.Enable(False)
+	#------------------------------------------------------------
+	def set_selections(self, selections = None):
+		self._LCTRL_left.set_selections(selections = selections)
+	#------------------------------------------------------------
+	def set_choices(self, choices=None, data=None):
+		self.set_string_items(items = choices)
+		if data is not None:
+			self.set_data(data = data)
+	#------------------------------------------------------------
+	def set_picks(self, picks=None, data=None):
+		self._LCTRL_right.set_string_items(picks)
+		self._LCTRL_right.set_column_widths()
+		if data is not None:
+			self._LCTRL_right.set_data(data = data)
+	#------------------------------------------------------------
+	def set_data(self, data = None):
+		self._LCTRL_left.set_data(data = data)
+	#------------------------------------------------------------
+	def get_picks(self):
+		return self._LCTRL_right.get_item_data()
+	#------------------------------------------------------------
+	# internal helpers
+	#------------------------------------------------------------
+	def __pick_selected(self, event=None):
+		if self._LCTRL_left.get_selected_items(only_one = True) == -1:
+			return
+
+		right_items = self._LCTRL_right.get_string_items()
+		right_data = self._LCTRL_right.get_item_data()
+
+		right_items.extend(self._LCTRL_left.get_selected_string_items(only_one = False))
+		self._LCTRL_right.set_string_items(items = right_items)
+		del right_items
+
+		if right_data is None:
+			self._LCTRL_right.set_data(data = self._LCTRL_left.get_selected_item_data(only_one = False))
+		else:
+			right_data.extend(self._LCTRL_left.get_selected_item_data(only_one = False))
+			self._LCTRL_right.set_data(data = right_data)
+		del right_data
+
+		self._LCTRL_right.set_column_widths()
+	#------------------------------------------------------------
+	def __remove_selected_picks(self):
+		if self._LCTRL_right.get_selected_items(only_one = True) == -1:
+			return
+
+		for item_idx in self._LCTRL_right.get_selected_items(only_one = False):
+			self._LCTRL_right.remove_item(item_idx)
+
+		if self._LCTRL_right.GetItemCount() == 0:
+			self._BTN_right2left.Enable(False)
+	#------------------------------------------------------------
+	# event handlers
+	#------------------------------------------------------------
+	def _on_left_list_item_selected(self, event):
+		self._BTN_left2right.Enable(True)
+	#------------------------------------------------------------
+	def _on_left_list_item_deselected(self, event):
+		if self._LCTRL_left.get_selected_items(only_one = True) == -1:
+			self._BTN_left2right.Enable(False)
+	#------------------------------------------------------------
+	def _on_right_list_item_selected(self, event):
+		self._BTN_right2left.Enable(True)
+	#------------------------------------------------------------
+	def _on_right_list_item_deselected(self, event):
+		if self._LCTRL_right.get_selected_items(only_one = True) == -1:
+			self._BTN_right2left.Enable(False)
+	#------------------------------------------------------------
+	def _on_button_left2right_pressed(self, event):
+		self.__pick_selected()
+	#------------------------------------------------------------
+	def _on_button_right2left_pressed(self, event):
+		self.__remove_selected_picks()
+#================================================================
 class cReportListCtrl(wx.ListCtrl, listmixins.ListCtrlAutoWidthMixin):
 
+	# FIXME: searching by typing
+
 	def __init__(self, *args, **kwargs):
 
 		try:
@@ -488,6 +678,14 @@ class cReportListCtrl(wx.ListCtrl, listmixins.ListCtrlAutoWidthMixin):
 
 		self.__widths = None
 		self.__data = None
+		self.__activate_callback = None
+
+		self.Bind(wx.EVT_MOTION, self._on_mouse_motion)
+		self.__item_tooltip_callback = None
+		self.__tt_last_item = None
+		self.__tt_static_part = _("""Select the items you want to work on.
+
+A discontinuous selection may depend on your holding down a platform-dependent modifier key (<ctrl>, <alt>, etc) or key combination (eg. <ctrl-shift> or <ctrl-alt>) while clicking.""")
 	#------------------------------------------------------------
 	# setters
 	#------------------------------------------------------------
@@ -497,6 +695,7 @@ class cReportListCtrl(wx.ListCtrl, listmixins.ListCtrlAutoWidthMixin):
 		Note that this will (have to) delete the items.
 		"""
 		self.ClearAll()
+		self.__tt_last_item = None
 		if columns is None:
 			return
 		for idx in range(len(columns)):
@@ -538,6 +737,7 @@ class cReportListCtrl(wx.ListCtrl, listmixins.ListCtrlAutoWidthMixin):
 
 		self.DeleteAllItems()
 		self.__data = items
+		self.__tt_last_item = None
 
 		if items is None:
 			return
@@ -568,11 +768,13 @@ class cReportListCtrl(wx.ListCtrl, listmixins.ListCtrlAutoWidthMixin):
 	def set_data(self, data = None):
 		"""<data must be a list corresponding to the item indices>"""
 		self.__data = data
+		self.__tt_last_item = None
 	#------------------------------------------------------------
 	def set_selections(self, selections=None):
 		self.Select(0, on = 0)
 		for idx in selections:
 			self.Select(idx = idx, on = 1)
+			#self.SetItemState(idx, wx.LIST_STATE_SELECTED, wx.LIST_STATE_SELECTED)
 	#------------------------------------------------------------
 	# getters
 	#------------------------------------------------------------
@@ -583,11 +785,15 @@ class cReportListCtrl(wx.ListCtrl, listmixins.ListCtrlAutoWidthMixin):
 			labels.append(col.GetText())
 		return labels
 	#------------------------------------------------------------
-	def get_item_data(self, item_idx = None):
-		if self.__data is None:	# this isn't entirely clean
-			return None
-
-		return self.__data[item_idx]
+	def get_item(self, item_idx=None):
+		if item_idx is not None:
+			return self.GetItem(item_idx)
+	#------------------------------------------------------------
+	def get_items(self):
+		return [ self.GetItem(item_idx) for item_idx in range(self.GetItemCount()) ]
+	#------------------------------------------------------------
+	def get_string_items(self):
+		return [ self.GetItemText(item_idx) for item_idx in range(self.GetItemCount()) ]
 	#------------------------------------------------------------
 	def get_selected_items(self, only_one=False):
 
@@ -602,6 +808,28 @@ class cReportListCtrl(wx.ListCtrl, listmixins.ListCtrlAutoWidthMixin):
 
 		return items
 	#------------------------------------------------------------
+	def get_selected_string_items(self, only_one=False):
+
+		if self.__is_single_selection or only_one:
+			return self.GetItemText(self.GetFirstSelected())
+
+		items = []
+		idx = self.GetFirstSelected()
+		while idx != -1:
+			items.append(self.GetItemText(idx))
+			idx = self.GetNextSelected(idx)
+
+		return items
+	#------------------------------------------------------------
+	def get_item_data(self, item_idx = None):
+		if self.__data is None:	# this isn't entirely clean
+			return None
+
+		if item_idx is not None:
+			return self.__data[item_idx]
+
+		return [ self.__data[item_idx] for item_idx in range(self.GetItemCount()) ]
+	#------------------------------------------------------------
 	def get_selected_item_data(self, only_one=False):
 
 		if self.__is_single_selection or only_one:
@@ -624,11 +852,76 @@ class cReportListCtrl(wx.ListCtrl, listmixins.ListCtrlAutoWidthMixin):
 	#------------------------------------------------------------
 	def deselect_selected_item(self):
 		self.Select(idx = self.GetFirstSelected(), on = 0)
+	#------------------------------------------------------------
+	def remove_item(self, item_idx=None):
+		self.DeleteItem(item_idx)
+		if self.__data is not None:
+			del self.__data[item_idx]
+		self.__tt_last_item = None
+	#------------------------------------------------------------
+	# event handlers
+	#------------------------------------------------------------
+	def _on_list_item_activated(self, event):
+		event.Skip()
+		if self.__activate_callback is not None:
+			self.__activate_callback(event)
+	#------------------------------------------------------------
+	def _on_mouse_motion(self, event):
+		item_idx, where = self.HitTest(wx.Point(event.X, event.Y))
+
+		if self.__tt_last_item == item_idx:
+			return
+
+		self.__tt_last_item = item_idx
+
+		if item_idx == -1:
+			self.SetToolTipString(self.__tt_static_part)
+			return
+
+		dyna_tt = None
+		if self.__item_tooltip_callback is not None:
+			dyna_tt = self.__item_tooltip_callback(self.__data[item_idx])
+
+		if dyna_tt is None:
+			self.SetToolTipString(self.__tt_static_part)
+			return
+
+		self.SetToolTipString(dyna_tt)
+	#------------------------------------------------------------
+	# properties
+	#------------------------------------------------------------
+	def _get_activate_callback(self):
+		return self.__activate_callback
+
+	def _set_activate_callback(self, callback):
+		if callback is None:
+			self.Unbind(wx.EVT_LIST_ITEM_ACTIVATED)
+		else:
+			if not callable(callback):
+				raise ValueError('<activate> callback is not a callable: %s' % callback)
+			self.Bind(wx.EVT_LIST_ITEM_ACTIVATED, self._on_list_item_activated)
+		self.__activate_callback = callback
+
+	activate_callback = property(_get_activate_callback, _set_activate_callback)
+	#------------------------------------------------------------
+	def _set_item_tooltip_callback(self, callback):
+		if callback is not None:
+			if not callable(callback):
+				raise ValueError('<item_tooltip> callback is not a callable: %s' % callback)
+		self.__item_tooltip_callback = callback
+
+	item_tooltip_callback = property(lambda x:x, _set_item_tooltip_callback)
 #================================================================
 # main
 #----------------------------------------------------------------
 if __name__ == '__main__':
 
+	if len(sys.argv) < 2:
+		sys.exit()
+
+	if sys.argv[1] != 'test':
+		sys.exit()
+
 	from Gnumed.pycommon import gmI18N
 	gmI18N.activate_locale()
 	gmI18N.install_domain()
@@ -671,137 +964,19 @@ if __name__ == '__main__':
 		print "chosen:"
 		print chosen
 	#------------------------------------------------------------
-	if (len(sys.argv) > 1) and (sys.argv[1] == 'test'):
-		test_get_choices_from_list()
-		#test_wxMultiChoiceDialog()
+	def test_item_picker_dlg():
+		app = wx.PyWidgetTester(size = (200, 50))
+		dlg = cItemPickerDlg(None, -1, msg = 'Pick a few items:')
+		dlg.set_columns(['Plugins'], ['Load in workplace', 'dummy'])
+		#dlg.set_columns(['Plugins'], [])
+		dlg.set_string_items(['patient', 'emr', 'docs'])
+		result = dlg.ShowModal()
+		print result
+		print dlg.get_picks()
+	#------------------------------------------------------------
+	#test_get_choices_from_list()
+	#test_wxMultiChoiceDialog()
+	test_item_picker_dlg()
 
 #================================================================
-# $Log: gmListWidgets.py,v $
-# Revision 1.37  2010-02-06 21:37:26  ncq
-# - support ignoring OK button
-#
-# Revision 1.36  2010/01/31 18:17:33  ncq
-# - make refresh callback setting smarter: set column widths after setting items
-#
-# Revision 1.35  2010/01/21 08:43:23  ncq
-# - somewhat support setting col widths within get-choice-from-list
-#
-# Revision 1.34  2009/11/28 18:30:07  ncq
-# - hide disabled buttons/show enabled ones
-#
-# Revision 1.33  2009/06/29 15:07:58  ncq
-# - disable edit/delete buttons when setting items to None or []
-#
-# Revision 1.32  2009/06/20 12:45:22  ncq
-# - call refresh from refresh property setter if callable and not None
-#
-# Revision 1.31  2009/06/11 12:37:25  ncq
-# - much simplified initial setup of list ctrls
-#
-# Revision 1.30  2009/06/04 16:32:01  ncq
-# - use refresh from init if available to simplify external setup code
-#
-# Revision 1.29  2009/04/16 12:49:47  ncq
-# - more sanity checks regarding action callbacks
-#
-# Revision 1.28  2009/01/17 23:07:29  ncq
-# - support remembering previous widths policy
-#
-# Revision 1.27  2009/01/15 11:39:59  ncq
-# - cleanup
-#
-# Revision 1.26  2008/12/25 16:55:36  ncq
-# - allow returniny empty list = no item selected if desired
-#
-# Revision 1.25  2008/08/06 13:22:14  ncq
-# - fix detection of item list type
-#
-# Revision 1.24  2008/07/24 14:00:18  ncq
-# - better comments
-# - resize columns after list refreshing in generic list selector
-# - differentiate between iterables and non-iterables by means of
-#   an exception rather than checking for type.ListType in set_string_items
-#
-# Revision 1.23  2008/05/31 16:33:07  ncq
-# - add TODO with URL
-#
-# Revision 1.22  2008/02/26 16:28:04  ncq
-# - when auto-setting col widths in lists w/o items use header as width ;-)
-#
-# Revision 1.21  2007/11/28 22:37:00  ncq
-# - robustify in the absence of selected values
-#
-# Revision 1.20  2007/11/23 23:34:39  ncq
-# - when explicitely setting the selections deselect the
-#   0th-index item selected by default
-#
-# Revision 1.19  2007/11/17 16:38:13  ncq
-# - cGenericListManagerPnl
-#
-# Revision 1.18  2007/10/08 12:56:02  ncq
-# - document callbacks
-# - protect against self.__data being None in get(_selected)_item_data()
-#
-# Revision 1.17  2007/09/24 18:37:08  ncq
-# - get_column_labels()
-#
-# Revision 1.16  2007/09/20 19:10:15  ncq
-# - carefully handle list item insertion - handle both list
-#   of lists and list of strings
-#
-# Revision 1.15  2007/09/07 22:38:04  ncq
-# - remove Fit() call since it's counterproductive for the list
-#
-# Revision 1.14  2007/09/02 20:54:26  ncq
-# - remove cruft
-# - support refresh_callback
-#
-# Revision 1.13  2007/08/31 23:05:05  ncq
-# - fix single selection list
-#
-# Revision 1.12  2007/08/29 14:41:54  ncq
-# - no more singular get_choice_from_list()
-# - support add/delete callbacks in generic list selector
-#
-# Revision 1.11  2007/08/20 16:22:51  ncq
-# - make get_choice(s)_from_list() more generic
-# - cleanup, improved test
-# - support edit button and message in generic list selector
-#
-# Revision 1.10  2007/07/22 09:26:25  ncq
-# - new get_choice_from_list()
-#
-# Revision 1.9  2007/07/09 12:45:47  ncq
-# - fix unicode()ing in set_string_items(): can't use (..., errors='replace') :-(
-# - factor out cPatientListingCtrl into gmDataMiningWidgets.py
-#
-# Revision 1.8  2007/07/07 12:42:00  ncq
-# - set_string_items now applies unicode() to all item members
-# - cPatientListingCtrl and test suite
-#
-# Revision 1.7  2007/06/28 12:38:15  ncq
-# - fix logic reversal in get_selected_*()
-#
-# Revision 1.6  2007/06/18 20:33:56  ncq
-# - add get_choice(s)_from_list()
-# - add cGenericListSelectorDlg
-# - add set_string_items()/set_selections()/get_selected_items()
-# - improve test suite
-#
-# Revision 1.5  2007/06/12 16:03:02  ncq
-# - properly get rid of all columns in set_columns()
-#
-# Revision 1.4  2007/04/09 18:51:47  ncq
-# - add support for multiple selections and auto-setting the widths
-#
-# Revision 1.3  2007/03/18 14:09:31  ncq
-# - add set_columns() and set_column_widths()
-#
-# Revision 1.2  2006/12/11 20:50:45  ncq
-# - get_selected_item_data()
-# - deselect_selected_item()
-#
-# Revision 1.1  2006/07/23 20:34:50  ncq
-# - list controls and widgets
-#
 #
\ No newline at end of file
