@@ -7,18 +7,34 @@ import sys, logging, os, os.path, time, re as regex, shutil
 
 
 import wx
-import wx.lib.expando as wxexpando
+import wx.lib.expando as wx_expando
+import wx.lib.agw.supertooltip as agw_stt
+import wx.lib.statbmp as wx_genstatbmp
 
 
 if __name__ == '__main__':
 	sys.path.insert(0, '../../')
-from Gnumed.pycommon import gmI18N, gmDispatcher, gmTools, gmDateTime
-from Gnumed.pycommon import gmShellAPI, gmPG2, gmCfg, gmMatchProvider
+from Gnumed.pycommon import gmI18N
+from Gnumed.pycommon import gmDispatcher
+from Gnumed.pycommon import gmTools
+from Gnumed.pycommon import gmDateTime
+from Gnumed.pycommon import gmShellAPI
+from Gnumed.pycommon import gmPG2
+from Gnumed.pycommon import gmCfg
+from Gnumed.pycommon import gmMatchProvider
+
 from Gnumed.business import gmPerson, gmEMRStructItems, gmClinNarrative, gmSurgery
-from Gnumed.business import gmForms, gmDocuments
-from Gnumed.wxpython import gmListWidgets, gmEMRStructWidgets, gmRegetMixin
-from Gnumed.wxpython import gmPhraseWheel, gmGuiHelpers, gmPatSearchWidgets
-from Gnumed.wxpython import gmCfgWidgets, gmDocumentWidgets
+from Gnumed.business import gmForms, gmDocuments, gmPersonSearch
+
+from Gnumed.wxpython import gmListWidgets
+from Gnumed.wxpython import gmEMRStructWidgets
+from Gnumed.wxpython import gmRegetMixin
+from Gnumed.wxpython import gmPhraseWheel
+from Gnumed.wxpython import gmGuiHelpers
+from Gnumed.wxpython import gmPatSearchWidgets
+from Gnumed.wxpython import gmCfgWidgets
+from Gnumed.wxpython import gmDocumentWidgets
+
 from Gnumed.exporters import gmPatientExporter
 
 
@@ -392,6 +408,91 @@ def export_narrative_for_medistar_import(parent=None, soap_cats=u'soap', encount
 	wx.EndBusyCursor()
 	return True
 #------------------------------------------------------------
+def select_narrative_from_episodes_new(parent=None, soap_cats=None):
+	"""soap_cats needs to be a list"""
+
+	if parent is None:
+		parent = wx.GetApp().GetTopWindow()
+
+	pat = gmPerson.gmCurrentPatient()
+	emr = pat.get_emr()
+
+	selected_soap = {}
+	selected_narrative_pks = []
+
+	#-----------------------------------------------
+	def pick_soap_from_episode(episode):
+
+		narr_for_epi = emr.get_clin_narrative(episodes = [episode['pk_episode']], soap_cats = soap_cats)
+
+		if len(narr_for_epi) == 0:
+			gmDispatcher.send(signal = 'statustext', msg = _('No narrative available for selected episode.'))
+			return True
+
+		dlg = cNarrativeListSelectorDlg (
+			parent = parent,
+			id = -1,
+			narrative = narr_for_epi,
+			msg = _(
+				'\n This is the narrative (type %s) for the chosen episodes.\n'
+				'\n'
+				' Now, mark the entries you want to include in your report.\n'
+			) % u'/'.join([ gmClinNarrative.soap_cat2l10n[cat] for cat in gmTools.coalesce(soap_cats, list(u'soap')) ])
+		)
+#		selection_idxs = []
+#		for idx in range(len(narr_for_epi)):
+#			if narr_for_epi[idx]['pk_narrative'] in selected_narrative_pks:
+#				selection_idxs.append(idx)
+#		if len(selection_idxs) != 0:
+#			dlg.set_selections(selections = selection_idxs)
+		btn_pressed = dlg.ShowModal()
+		selected_narr = dlg.get_selected_item_data()
+		dlg.Destroy()
+
+		if btn_pressed == wx.ID_CANCEL:
+			return True
+
+		selected_narrative_pks = [ i['pk_narrative'] for i in selected_narr ]
+		for narr in selected_narr:
+			selected_soap[narr['pk_narrative']] = narr
+
+		print "before returning from picking soap"
+
+		return True
+	#-----------------------------------------------
+	selected_episode_pks = []
+
+	all_epis = [ epi for epi in emr.get_episodes() if epi.has_narrative ]
+
+	if len(all_epis) == 0:
+		gmDispatcher.send(signal = 'statustext', msg = _('No episodes recorded for the health issues selected.'))
+		return []
+
+	dlg = gmEMRStructWidgets.cEpisodeListSelectorDlg (
+		parent = parent,
+		id = -1,
+		episodes = all_epis,
+		msg = _('\n Select the the episode you want to report on.\n')
+	)
+#	selection_idxs = []
+#	for idx in range(len(all_epis)):
+#		if all_epis[idx]['pk_episode'] in selected_episode_pks:
+#			selection_idxs.append(idx)
+#	if len(selection_idxs) != 0:
+#		dlg.set_selections(selections = selection_idxs)
+	dlg.left_extra_button = (
+		_('Pick SOAP'),
+		_('Pick SOAP entries from topmost selected episode'),
+		pick_soap_from_episode
+	)
+	btn_pressed = dlg.ShowModal()
+	dlg.Destroy()
+
+	if btn_pressed == wx.ID_CANCEL:
+		return None
+
+	return selected_soap.values()
+#------------------------------------------------------------
 def select_narrative_from_episodes(parent=None, soap_cats=None):
 	"""soap_cats needs to be a list"""
 
@@ -574,13 +675,13 @@ class cSoapPluginPnl(wxgSoapPluginPnl.wxgSoapPluginPnl, gmRegetMixin.cRegetOnPai
 
 	Left hand side:
 	- problem list (health issues and active episodes)
-	- hints area
+	- previous notes
 
 	Right hand side:
-	- previous notes
-	- notebook with progress note editors
 	- encounter details fields
-	- visual soap area
+	- notebook with progress note editors
+	- visual progress notes
+	- hints
 
 	Listens to patient change signals, thus acts on the current patient.
 	"""
@@ -590,6 +691,7 @@ class cSoapPluginPnl(wxgSoapPluginPnl.wxgSoapPluginPnl, gmRegetMixin.cRegetOnPai
 		gmRegetMixin.cRegetOnPaintMixin.__init__(self)
 
 		self.__pat = gmPerson.gmCurrentPatient()
+		self.__patient_just_changed = False
 		self.__init_ui()
 		self.__reset_ui_content()
 
@@ -626,13 +728,13 @@ class cSoapPluginPnl(wxgSoapPluginPnl.wxgSoapPluginPnl, gmRegetMixin.cRegetOnPai
 	# internal helpers
 	#--------------------------------------------------------
 	def __init_ui(self):
-		self._LCTRL_active_problems.set_columns([_('Last'), _('Problem'), _('Health issue')])
+		self._LCTRL_active_problems.set_columns([_('Last'), _('Problem'), _('In health issue')])
 		self._LCTRL_active_problems.set_string_items()
 
 		self._splitter_main.SetSashGravity(0.5)
 		self._splitter_left.SetSashGravity(0.5)
 		self._splitter_right.SetSashGravity(1.0)
-		self._splitter_soap.SetSashGravity(0.75)
+#		self._splitter_soap.SetSashGravity(0.75)
 
 		splitter_size = self._splitter_main.GetSizeTuple()[0]
 		self._splitter_main.SetSashPosition(splitter_size * 3 / 10, True)
@@ -643,15 +745,14 @@ class cSoapPluginPnl(wxgSoapPluginPnl.wxgSoapPluginPnl, gmRegetMixin.cRegetOnPai
 		splitter_size = self._splitter_right.GetSizeTuple()[1]
 		self._splitter_right.SetSashPosition(splitter_size * 15 / 20, True)
 
-		splitter_size = self._splitter_soap.GetSizeTuple()[0]
-		self._splitter_soap.SetSashPosition(splitter_size * 3 / 4, True)
+#		splitter_size = self._splitter_soap.GetSizeTuple()[0]
+#		self._splitter_soap.SetSashPosition(splitter_size * 3 / 4, True)
 
 		self._NB_soap_editors.DeleteAllPages()
 	#--------------------------------------------------------
 	def __reset_ui_content(self):
-		"""
-		Clear all information from input panel
-		"""
+		"""Clear all information from input panel."""
+
 		self._LCTRL_active_problems.set_string_items()
 
 		self._TCTRL_recent_notes.SetValue(u'')
@@ -665,16 +766,10 @@ class cSoapPluginPnl(wxgSoapPluginPnl.wxgSoapPluginPnl, gmRegetMixin.cRegetOnPai
 		self._NB_soap_editors.DeleteAllPages()
 		self._NB_soap_editors.add_editor()
 
-		self._PNL_visual_soap.clear()
-
 		self._lbl_hints.SetLabel(u'')
 	#--------------------------------------------------------
-	def __refresh_visual_soaps(self):
-		self._PNL_visual_soap.refresh()
-	#--------------------------------------------------------
 	def __refresh_problem_list(self):
-		"""Update health problems list.
-		"""
+		"""Update health problems list."""
 
 		self._LCTRL_active_problems.set_string_items()
 
@@ -701,7 +796,7 @@ class cSoapPluginPnl(wxgSoapPluginPnl.wxgSoapPluginPnl, gmRegetMixin.cRegetOnPai
 				else:
 					last = last_encounter['last_affirmed'].strftime('%m/%Y')
 
-				list_items.append([last, problem['problem'], gmTools.u_left_arrow])
+				list_items.append([last, problem['problem'], gmTools.u_down_left_arrow])		#gmTools.u_left_arrow
 
 			elif problem['type'] == 'episode':
 				epi = emr.problem2episode(problem)
@@ -714,7 +809,7 @@ class cSoapPluginPnl(wxgSoapPluginPnl.wxgSoapPluginPnl, gmRegetMixin.cRegetOnPai
 				list_items.append ([
 					last,
 					problem['problem'],
-					gmTools.coalesce(initial = epi['health_issue'], instead = gmTools.u_diameter)
+					gmTools.coalesce(initial = epi['health_issue'], instead = u'?')		#gmTools.u_diameter
 				])
 
 		self._LCTRL_active_problems.set_string_items(items = list_items)
@@ -733,78 +828,133 @@ class cSoapPluginPnl(wxgSoapPluginPnl.wxgSoapPluginPnl, gmRegetMixin.cRegetOnPai
 
 		return True
 	#--------------------------------------------------------
+	def __get_soap_for_issue_problem(self, problem=None):
+		soap = u''
+		emr = self.__pat.get_emr()
+		prev_enc = emr.get_last_but_one_encounter(issue_id = problem['pk_health_issue'])
+		if prev_enc is not None:
+			soap += prev_enc.format (
+				issues = [ problem['pk_health_issue'] ],
+				with_soap = True,
+				with_docs = False,
+				with_tests = False,
+				patient = self.__pat,
+				fancy_header = False,
+				with_rfe_aoe = True
+			)
+
+		tmp = emr.active_encounter.format_soap (
+			soap_cats = 'soap',
+			emr = emr,
+			issues = [ problem['pk_health_issue'] ],
+		)
+		if len(tmp) > 0:
+			soap += _('Current encounter:') + u'\n'
+			soap += u'\n'.join(tmp) + u'\n'
+
+		if problem['summary'] is not None:
+			soap += u'\n-- %s ----------\n%s' % (
+				_('Cumulative summary'),
+				gmTools.wrap (
+					text = problem['summary'],
+					width = 45,
+					initial_indent = u' ',
+					subsequent_indent = u' '
+				).strip('\n')
+			)
+
+		return soap
+	#--------------------------------------------------------
+	def __get_soap_for_episode_problem(self, problem=None):
+		soap = u''
+		emr = self.__pat.get_emr()
+		prev_enc = emr.get_last_but_one_encounter(episode_id = problem['pk_episode'])
+		if prev_enc is not None:
+			soap += prev_enc.format (
+				episodes = [ problem['pk_episode'] ],
+				with_soap = True,
+				with_docs = False,
+				with_tests = False,
+				patient = self.__pat,
+				fancy_header = False,
+				with_rfe_aoe = True
+			)
+		else:
+			if problem['pk_health_issue'] is not None:
+				prev_enc = emr.get_last_but_one_encounter(episode_id = problem['pk_health_issue'])
+				if prev_enc is not None:
+					soap += prev_enc.format (
+						with_soap = True,
+						with_docs = False,
+						with_tests = False,
+						patient = self.__pat,
+						issues = [ problem['pk_health_issue'] ],
+						fancy_header = False,
+						with_rfe_aoe = True
+					)
+
+		tmp = emr.active_encounter.format_soap (
+			soap_cats = 'soap',
+			emr = emr,
+			issues = [ problem['pk_health_issue'] ],
+		)
+		if len(tmp) > 0:
+			soap += _('Current encounter:') + u'\n'
+			soap += u'\n'.join(tmp) + u'\n'
+
+		if problem['summary'] is not None:
+			soap += u'\n-- %s ----------\n%s' % (
+				_('Cumulative summary'),
+				gmTools.wrap (
+					text = problem['summary'],
+					width = 45,
+					initial_indent = u' ',
+					subsequent_indent = u' '
+				).strip('\n')
+			)
+
+		return soap
+	#--------------------------------------------------------
+	def __refresh_current_editor(self):
+		self._NB_soap_editors.refresh_current_editor()
+	#--------------------------------------------------------
+	def __setup_initial_patient_editors(self):
+		if not self.__patient_just_changed:
+			return
+
+		dbcfg = gmCfg.cCfgSQL()
+		auto_open_recent_problems = bool(dbcfg.get2 (
+			option = u'horstspace.soap_editor.auto_open_latest_episodes',
+			workplace = gmSurgery.gmCurrentPractice().active_workplace,
+			bias = u'user',
+			default = True
+		))
+
+		self.__patient_just_changed = False
+		emr = self.__pat.get_emr()
+		recent_epis = emr.active_encounter.get_episodes()
+		prev_enc = emr.get_last_but_one_encounter()
+		if prev_enc is not None:
+			recent_epis.extend(prev_enc.get_episodes())
+
+		for epi in recent_epis:
+			if not epi['episode_open']:
+				continue
+			self._NB_soap_editors.add_editor(problem = epi, allow_same_problem = False)
+	#--------------------------------------------------------
 	def __refresh_recent_notes(self, problem=None):
 		"""This refreshes the recent-notes part."""
 
-		if problem is None:
-			soap = u''
-			caption = u'<?>'
+		soap = u''
+		caption = u'<?>'
 
-		elif problem['type'] == u'issue':
-			emr = self.__pat.get_emr()
-			soap = u''
+		if problem['type'] == u'issue':
 			caption = problem['problem'][:35]
-
-			prev_enc = emr.get_last_but_one_encounter(issue_id = problem['pk_health_issue'])
-			if prev_enc is not None:
-				soap += prev_enc.format (
-					with_soap = True,
-					with_docs = False,
-					with_tests = False,
-					patient = self.__pat,
-					issues = [ problem['pk_health_issue'] ],
-					fancy_header = False
-				)
-
-			tmp = emr.active_encounter.format_soap (
-				soap_cats = 'soap',
-				emr = emr,
-				issues = [ problem['pk_health_issue'] ],
-			)
-			if len(tmp) > 0:
-				soap += _('Current encounter:') + u'\n'
-				soap += u'\n'.join(tmp) + u'\n'
+			soap = self.__get_soap_for_issue_problem(problem = problem)
 
 		elif problem['type'] == u'episode':
-			emr = self.__pat.get_emr()
-			soap = u''
 			caption = problem['problem'][:35]
-
-			prev_enc = emr.get_last_but_one_encounter(episode_id = problem['pk_episode'])
-			if prev_enc is None:
-				if problem['pk_health_issue'] is not None:
-					prev_enc = emr.get_last_but_one_encounter(episode_id = problem['pk_health_issue'])
-					if prev_enc is not None:
-						soap += prev_enc.format (
-							with_soap = True,
-							with_docs = False,
-							with_tests = False,
-							patient = self.__pat,
-							issues = [ problem['pk_health_issue'] ],
-							fancy_header = False
-						)
-			else:
-				soap += prev_enc.format (
-					episodes = [ problem['pk_episode'] ],
-					with_soap = True,
-					with_docs = False,
-					with_tests = False,
-					patient = self.__pat,
-					fancy_header = False
-				)
-
-			tmp = emr.active_encounter.format_soap (
-				soap_cats = 'soap',
-				emr = emr,
-				issues = [ problem['pk_health_issue'] ],
-			)
-			if len(tmp) > 0:
-				soap += _('Current encounter:') + u'\n'
-				soap += u'\n'.join(tmp) + u'\n'
-
-		else:
-			soap = u''
-			caption = u'<?>'
+			soap = self.__get_soap_for_episode_problem(problem = problem)
 
 		self._TCTRL_recent_notes.SetValue(soap)
 		self._TCTRL_recent_notes.ShowPosition(self._TCTRL_recent_notes.GetLastPosition())
@@ -819,8 +969,8 @@ class cSoapPluginPnl(wxgSoapPluginPnl.wxgSoapPluginPnl, gmRegetMixin.cRegetOnPai
 		return True
 	#--------------------------------------------------------
 	def __refresh_encounter(self):
-		"""Update encounter fields.
-		"""
+		"""Update encounter fields."""
+
 		emr = self.__pat.get_emr()
 		enc = emr.active_encounter
 		self._PRW_encounter_type.SetText(value = enc['l10n_type'], data = enc['pk_type'])
@@ -964,9 +1114,10 @@ class cSoapPluginPnl(wxgSoapPluginPnl.wxgSoapPluginPnl, gmRegetMixin.cRegetOnPai
 	#--------------------------------------------------------
 	def _on_post_patient_selection(self):
 		wx.CallAfter(self._schedule_data_reget)
+		self.__patient_just_changed = True
 	#--------------------------------------------------------
 	def _on_doc_mod_db(self):
-		wx.CallAfter(self.__refresh_visual_soaps)
+		wx.CallAfter(self.__refresh_current_editor)
 	#--------------------------------------------------------
 	def _on_episode_issue_mod_db(self):
 		wx.CallAfter(self._schedule_data_reget)
@@ -979,7 +1130,8 @@ class cSoapPluginPnl(wxgSoapPluginPnl.wxgSoapPluginPnl, gmRegetMixin.cRegetOnPai
 	#--------------------------------------------------------
 	def __on_current_encounter_switched(self):
 		self.__refresh_encounter()
-		self.__refresh_visual_soaps()
+	#--------------------------------------------------------
+	# problem list specific events
 	#--------------------------------------------------------
 	def _on_problem_focused(self, event):
 		"""Show related note at the bottom."""
@@ -1019,6 +1171,14 @@ class cSoapPluginPnl(wxgSoapPluginPnl.wxgSoapPluginPnl, gmRegetMixin.cRegetOnPai
 		event.Skip()
 		return False
 	#--------------------------------------------------------
+	def _on_show_closed_episodes_checked(self, event):
+		self.__refresh_problem_list()
+	#--------------------------------------------------------
+	def _on_irrelevant_issues_checked(self, event):
+		self.__refresh_problem_list()
+	#--------------------------------------------------------
+	# SOAP editor specific buttons
+	#--------------------------------------------------------
 	def _on_discard_editor_button_pressed(self, event):
 		self._NB_soap_editors.close_current_editor()
 		event.Skip()
@@ -1031,31 +1191,6 @@ class cSoapPluginPnl(wxgSoapPluginPnl.wxgSoapPluginPnl, gmRegetMixin.cRegetOnPai
 		self._NB_soap_editors.clear_current_editor()
 		event.Skip()
 	#--------------------------------------------------------
-	def _on_save_all_button_pressed(self, event):
-		self.save_encounter()
-		time.sleep(0.3)
-		event.Skip()
-		wx.SafeYield()
-
-		wx.CallAfter(self._save_all_button_pressed_bottom_half)
-		wx.SafeYield()
-	#--------------------------------------------------------
-	def _save_all_button_pressed_bottom_half(self):
-		emr = self.__pat.get_emr()
-		saved = self._NB_soap_editors.save_all_editors (
-			emr = emr,
-			episode_name_candidates = [
-				gmTools.none_if(self._TCTRL_aoe.GetValue().strip(), u''),
-				gmTools.none_if(self._TCTRL_rfe.GetValue().strip(), u'')
-			]
-		)
-		if not saved:
-			gmDispatcher.send(signal = 'statustext', msg = _('Cannot save all editors. Some were kept open.'), beep = True)
-	#--------------------------------------------------------
-	def _on_save_encounter_button_pressed(self, event):
-		self.save_encounter()
-		event.Skip()
-	#--------------------------------------------------------
 	def _on_save_note_button_pressed(self, event):
 		emr = self.__pat.get_emr()
 		self._NB_soap_editors.save_current_editor (
@@ -1067,6 +1202,37 @@ class cSoapPluginPnl(wxgSoapPluginPnl.wxgSoapPluginPnl, gmRegetMixin.cRegetOnPai
 		)
 		event.Skip()
 	#--------------------------------------------------------
+	def _on_save_note_under_button_pressed(self, event):
+		encounter = gmEMRStructWidgets.select_encounters (
+			parent = self,
+			patient = self.__pat,
+			single_selection = True
+		)
+		if encounter is None:
+			return
+
+		emr = self.__pat.get_emr()
+		self._NB_soap_editors.save_current_editor (
+			emr = emr,
+			encounter = encounter['pk_encounter'],
+			episode_name_candidates = [
+				gmTools.none_if(self._TCTRL_aoe.GetValue().strip(), u''),
+				gmTools.none_if(self._TCTRL_rfe.GetValue().strip(), u'')
+			]
+		)
+		event.Skip()
+	#--------------------------------------------------------
+	def _on_image_button_pressed(self, event):
+		emr = self.__pat.get_emr()
+		self._NB_soap_editors.add_visual_progress_note_to_current_problem()
+		event.Skip()
+	#--------------------------------------------------------
+	# encounter specific buttons
+	#--------------------------------------------------------
+	def _on_save_encounter_button_pressed(self, event):
+		self.save_encounter()
+		event.Skip()
+	#--------------------------------------------------------
 	def _on_new_encounter_button_pressed(self, event):
 
 		if self.__encounter_modified():
@@ -1091,18 +1257,35 @@ class cSoapPluginPnl(wxgSoapPluginPnl.wxgSoapPluginPnl, gmRegetMixin.cRegetOnPai
 
 		wx.CallAfter(gmEMRStructWidgets.start_new_encounter, emr = emr)
 	#--------------------------------------------------------
-	def _on_show_closed_episodes_checked(self, event):
-		self.__refresh_problem_list()
+	# other buttons
 	#--------------------------------------------------------
-	def _on_irrelevant_issues_checked(self, event):
-		self.__refresh_problem_list()
+	def _on_save_all_button_pressed(self, event):
+		self.save_encounter()
+		time.sleep(0.3)
+		event.Skip()
+		wx.SafeYield()
+
+		wx.CallAfter(self._save_all_button_pressed_bottom_half)
+		wx.SafeYield()
+	#--------------------------------------------------------
+	def _save_all_button_pressed_bottom_half(self):
+		emr = self.__pat.get_emr()
+		saved = self._NB_soap_editors.save_all_editors (
+			emr = emr,
+			episode_name_candidates = [
+				gmTools.none_if(self._TCTRL_aoe.GetValue().strip(), u''),
+				gmTools.none_if(self._TCTRL_rfe.GetValue().strip(), u'')
+			]
+		)
+		if not saved:
+			gmDispatcher.send(signal = 'statustext', msg = _('Cannot save all editors. Some were kept open.'), beep = True)
 	#--------------------------------------------------------
 	# reget mixin API
 	#--------------------------------------------------------
 	def _populate_with_data(self):
 		self.__refresh_problem_list()
 		self.__refresh_encounter()
-		self.__refresh_visual_soaps()
+		self.__setup_initial_patient_editors()
 		return True
 #============================================================
 class cSoapNoteInputNotebook(wx.Notebook):
@@ -1215,12 +1398,12 @@ class cSoapNoteInputNotebook(wx.Notebook):
 		if self.GetPageCount() == 0:
 			self.add_editor()
 	#--------------------------------------------------------
-	def save_current_editor(self, emr=None, episode_name_candidates=None):
+	def save_current_editor(self, emr=None, episode_name_candidates=None, encounter=None):
 
 		page_idx = self.GetSelection()
 		page = self.GetPage(page_idx)
 
-		if not page.save(emr = emr, episode_name_candidates = episode_name_candidates):
+		if not page.save(emr = emr, episode_name_candidates = episode_name_candidates, encounter = encounter):
 			return
 
 		self.DeletePage(page_idx)
@@ -1278,10 +1461,38 @@ class cSoapNoteInputNotebook(wx.Notebook):
 		page_idx = self.GetSelection()
 		page = self.GetPage(page_idx)
 		return page.problem
+	#--------------------------------------------------------
+	def refresh_current_editor(self):
+		page_idx = self.GetSelection()
+		page = self.GetPage(page_idx)
+		page.refresh()
+	#--------------------------------------------------------
+	def add_visual_progress_note_to_current_problem(self):
+		page_idx = self.GetSelection()
+		page = self.GetPage(page_idx)
+		page.add_visual_progress_note()
 #============================================================
 from Gnumed.wxGladeWidgets import wxgSoapNoteExpandoEditAreaPnl
 
 class cSoapNoteExpandoEditAreaPnl(wxgSoapNoteExpandoEditAreaPnl.wxgSoapNoteExpandoEditAreaPnl):
+	"""An Edit Area like panel for entering progress notes.
+
+	Subjective:
+		expando text ctrl
+	Objective:
+		expando text ctrl
+	Assessment:
+		expando text ctrl
+	Plan:
+		expando text ctrl
+	Problem summary:
+		text ctrl
+	visual progress notes
+		panel with images
+
+	- knows the problem this edit area is about
+	- can deal with issue or episode type problems
+	"""
 
 	def __init__(self, *args, **kwargs):
 
@@ -1300,18 +1511,83 @@ class cSoapNoteExpandoEditAreaPnl(wxgSoapNoteExpandoEditAreaPnl.wxgSoapNoteExpan
 			self._TCTRL_soaP
 		]
 
+		self.__init_ui()
 		self.__register_interests()
 	#--------------------------------------------------------
+	def __init_ui(self):
+		self.refresh_summary()
+		if self.problem is not None:
+			if self.problem['summary'] is None:
+				self._TCTRL_summary.SetValue(u'')
+		self.refresh_visual_soap()
+	#--------------------------------------------------------
+	def refresh(self):
+		self.refresh_summary()
+		self.refresh_visual_soap()
+	#--------------------------------------------------------
+	def refresh_summary(self):
+		if self.problem is None:
+			return
+		if self.problem['summary'] is None:
+			self._TCTRL_summary.SetValue(u'')
+		else:
+			self._TCTRL_summary.SetValue(self.problem['summary'])
+	#--------------------------------------------------------
+	def refresh_visual_soap(self):
+		if self.problem is None:
+			self._PNL_visual_soap.refresh(document_folder = None)
+			return
+
+		if self.problem['type'] == u'issue':
+			self._PNL_visual_soap.refresh(document_folder = None)
+			return
+
+		if self.problem['type'] == u'episode':
+			pat = gmPerson.gmCurrentPatient()
+			doc_folder = pat.get_document_folder()
+			emr = pat.get_emr()
+			self._PNL_visual_soap.refresh (
+				document_folder = doc_folder,
+				episodes = [self.problem['pk_episode']],
+				encounter = emr.active_encounter['pk_encounter']
+			)
+			return
+	#--------------------------------------------------------
 	def clear(self):
 		for field in self.fields:
 			field.SetValue(u'')
+		self._TCTRL_summary.SetValue(u'')
+		self._PNL_visual_soap.clear()
 	#--------------------------------------------------------
-	def save(self, emr=None, episode_name_candidates=None):
+	def add_visual_progress_note(self):
+		fname, discard_unmodified = select_visual_progress_note_template(parent = self)
+		if fname is None:
+			return False
+
+		if self.problem is None:
+			issue = None
+			episode = None
+		elif self.problem['type'] == 'issue':
+			issue = self.problem['pk_health_issue']
+			episode = None
+		else:
+			issue = self.problem['pk_health_issue']
+			episode = gmEMRStructItems.problem2episode(self.problem)
+
+		wx.CallAfter (
+			edit_visual_progress_note,
+			filename = fname,
+			episode = episode,
+			discard_unmodified = discard_unmodified,
+			health_issue = issue
+		)
+	#--------------------------------------------------------
+	def save(self, emr=None, episode_name_candidates=None, encounter=None):
 
 		if self.empty:
 			return True
 
-		# new unassociated episode
+		# new episode (standalone=unassociated or new-in-issue)
 		if (self.problem is None) or (self.problem['type'] == 'issue'):
 
 			episode_name_candidates.append(u'')
@@ -1339,6 +1615,8 @@ class cSoapNoteExpandoEditAreaPnl(wxgSoapNoteExpandoEditAreaPnl.wxgSoapNoteExpan
 
 			# create episode
 			new_episode = emr.add_episode(episode_name = epi_name[:45], pk_health_issue = None, is_open = True)
+			new_episode['summary'] = self._TCTRL_summary.GetValue().strip()
+			new_episode.save()
 
 			if self.problem is not None:
 				issue = emr.problem2issue(self.problem)
@@ -1361,10 +1639,20 @@ class cSoapNoteExpandoEditAreaPnl(wxgSoapNoteExpandoEditAreaPnl.wxgSoapNoteExpan
 					)
 
 			epi_id = new_episode['pk_episode']
+
+		# existing episode
 		else:
 			epi_id = self.problem['pk_episode']
 
-		emr.add_notes(notes = self.soap, episode = epi_id)
+		emr.add_notes(notes = self.soap, episode = epi_id, encounter = encounter)
+
+		# set summary but only if not already set above for an episode
+		# newly created either standalone or within a health issue
+		if self.problem is not None:
+			if self.problem['type'] == 'episode':
+				epi = emr.problem2episode(self.problem)
+				epi['summary'] = self._TCTRL_summary.GetValue().strip()
+				epi.save()
 
 		return True
 	#--------------------------------------------------------
@@ -1372,7 +1660,8 @@ class cSoapNoteExpandoEditAreaPnl(wxgSoapNoteExpandoEditAreaPnl.wxgSoapNoteExpan
 	#--------------------------------------------------------
 	def __register_interests(self):
 		for field in self.fields:
-			wxexpando.EVT_ETC_LAYOUT_NEEDED(field, field.GetId(), self._on_expando_needs_layout)
+			wx_expando.EVT_ETC_LAYOUT_NEEDED(field, field.GetId(), self._on_expando_needs_layout)
+		wx_expando.EVT_ETC_LAYOUT_NEEDED(self._TCTRL_summary, self._TCTRL_summary.GetId(), self._on_expando_needs_layout)
 	#--------------------------------------------------------
 	def _on_expando_needs_layout(self, evt):
 		# need to tell ourselves to re-Layout to refresh scroll bars
@@ -1436,15 +1725,28 @@ class cSoapNoteExpandoEditAreaPnl(wxgSoapNoteExpandoEditAreaPnl.wxgSoapNoteExpan
 		for field in self.fields:
 			if field.GetValue().strip() != u'':
 				return False
+
+		summary = self._TCTRL_summary.GetValue().strip()
+		if self.problem is None:
+			if summary != u'':
+				return False
+		else:
+			if self.problem['summary'] is None:
+				if summary != u'':
+					return False
+			else:
+				if summary != self.problem['summary'].strip():
+					return False
+
 		return True
 
 	empty = property(_get_empty, lambda x:x)
 #============================================================
-class cSoapLineTextCtrl(wxexpando.ExpandoTextCtrl):
+class cSoapLineTextCtrl(wx_expando.ExpandoTextCtrl):
 
 	def __init__(self, *args, **kwargs):
 
-		wxexpando.ExpandoTextCtrl.__init__(self, *args, **kwargs)
+		wx_expando.ExpandoTextCtrl.__init__(self, *args, **kwargs)
 
 		self.__keyword_separators = regex.compile("[!?'\".,:;)}\]\r\n\s\t]+")
 
@@ -1495,7 +1797,7 @@ class cSoapLineTextCtrl(wxexpando.ExpandoTextCtrl):
 		wx.CallAfter(self._after_on_focus)
 	#--------------------------------------------------------
 	def _after_on_focus(self):
-		evt = wx.PyCommandEvent(wxexpando.wxEVT_ETC_LAYOUT_NEEDED, self.GetId())
+		evt = wx.PyCommandEvent(wx_expando.wxEVT_ETC_LAYOUT_NEEDED, self.GetId())
 		evt.SetEventObject(self)
 		evt.height = None
 		evt.numLines = None
@@ -1585,9 +1887,6 @@ class cSoapLineTextCtrl(wxexpando.ExpandoTextCtrl):
 		return
 #============================================================
 # visual progress notes
-#============================================================
-visual_progress_note_document_type = u'visual progress note'
-
 #============================================================
 def configure_visual_progress_note_editor():
 
@@ -1635,7 +1934,62 @@ def configure_visual_progress_note_editor():
 		validator = is_valid
 	)
 #============================================================
-def edit_visual_progress_note(filename=None, episode=None, discard_unmodified=False, doc_part=None):
+def select_file_as_visual_progress_note_template(parent=None):
+	if parent is None:
+		parent = wx.GetApp().GetTopWindow()
+
+	dlg = wx.FileDialog (
+		parent = parent,
+		message = _('Choose file to use as template for new visual progress note'),
+		defaultDir = os.path.expanduser('~'),
+		defaultFile = '',
+		#wildcard = "%s (*)|*|%s (*.*)|*.*" % (_('all files'), _('all files (Win)')),
+		style = wx.OPEN | wx.HIDE_READONLY | wx.FILE_MUST_EXIST
+	)
+	result = dlg.ShowModal()
+
+	if result == wx.ID_CANCEL:
+		dlg.Destroy()
+		return None
+
+	full_filename = dlg.GetPath()
+	dlg.Hide()
+	dlg.Destroy()
+	return full_filename
+#------------------------------------------------------------
+def select_visual_progress_note_template(parent=None):
+
+	if parent is None:
+		parent = wx.GetApp().GetTopWindow()
+
+	# 1) select from template
+	from Gnumed.wxpython import gmFormWidgets
+	template = gmFormWidgets.manage_form_templates (
+		parent = parent,
+		template_types = [gmDocuments.DOCUMENT_TYPE_VISUAL_PROGRESS_NOTE],
+		active_only = True
+	)
+
+	# 2) select from disk file
+	if template is None:
+		fname = select_file_as_visual_progress_note_template(parent = parent)
+		if fname is None:
+			return (None, None)
+		# create a copy of the picked file -- don't modify the original
+		ext = os.path.splitext(fname)[1]
+		tmp_name = gmTools.get_unique_filename(suffix = ext)
+		_log.debug('visual progress note from file: [%s] -> [%s]', fname, tmp_name)
+		shutil.copy2(fname, tmp_name)
+		return (tmp_name, False)
+
+	filename = template.export_to_file()
+	if filename is None:
+		gmDispatcher.send(signal = u'statustext', msg = _('Cannot export visual progress note template for [%s].') % template['name_long'])
+		return (None, None)
+	return (filename, True)
+
+#------------------------------------------------------------
+def edit_visual_progress_note(filename=None, episode=None, discard_unmodified=False, doc_part=None, health_issue=None):
 	"""This assumes <filename> contains an image which can be handled by the configured image editor."""
 
 	if doc_part is not None:
@@ -1707,13 +2061,14 @@ def edit_visual_progress_note(filename=None, episode=None, discard_unmodified=Fa
 				_log.debug('visual progress note (template) not modified')
 				# ask user to decide
 				msg = _(
-					u'This visual progress note was created from a\n'
-					u'template in the database rather than from a file\n'
-					u'but the image was not modified at all.\n'
+					u'You either created a visual progress note from a template\n'
+					u'in the database (rather than from a file on disk) or you\n'
+					u'edited an existing visual progress note.\n'
 					u'\n'
-					u'Do you want to still save the unmodified\n'
-					u'image as a visual progress note into the\n'
-					u'EMR of the patient ?'
+					u'The template/original was not modified at all, however.\n'
+					u'\n'
+					u'Do you still want to save the unmodified image as a\n'
+					u'visual progress note into the EMR of the patient ?\n'
 				)
 				save_unmodified = gmGuiHelpers.gm_show_question (
 					msg,
@@ -1729,13 +2084,15 @@ def edit_visual_progress_note(filename=None, episode=None, discard_unmodified=Fa
 		return None
 
 	if not isinstance(episode, gmEMRStructItems.cEpisode):
+		if episode is None:
+			episode = _('visual progress notes')
 		pat = gmPerson.gmCurrentPatient()
 		emr = pat.get_emr()
-		episode = emr.add_episode(episode_name = episode.strip(), is_open = False)
+		episode = emr.add_episode(episode_name = episode.strip(), pk_health_issue = health_issue, is_open = False)
 
 	doc = gmDocumentWidgets.save_file_as_new_document (
 		filename = filename,
-		document_type = visual_progress_note_document_type,
+		document_type = gmDocuments.DOCUMENT_TYPE_VISUAL_PROGRESS_NOTE,
 		episode = episode,
 		unlock_patient = True
 	)
@@ -1764,7 +2121,7 @@ WHERE
 	)
 ORDER BY name_short
 LIMIT 15
-"""	% visual_progress_note_document_type
+"""	% gmDocuments.DOCUMENT_TYPE_VISUAL_PROGRESS_NOTE
 
 		mp = gmMatchProvider.cMatchProvider_SQL2(queries = [query])
 		mp.setThresholds(2, 3, 5)
@@ -1778,6 +2135,78 @@ LIMIT 15
 
 		return gmForms.cFormTemplate(aPK_obj = self.data)
 #============================================================
+from Gnumed.wxGladeWidgets import wxgVisualSoapPresenterPnl
+
+class cVisualSoapPresenterPnl(wxgVisualSoapPresenterPnl.wxgVisualSoapPresenterPnl):
+
+	def __init__(self, *args, **kwargs):
+		wxgVisualSoapPresenterPnl.wxgVisualSoapPresenterPnl.__init__(self, *args, **kwargs)
+		self._SZR_soap = self.GetSizer()
+		self.__bitmaps = []
+	#--------------------------------------------------------
+	# external API
+	#--------------------------------------------------------
+	def refresh(self, document_folder=None, episodes=None, encounter=None):
+
+		self.clear()
+		if document_folder is not None:
+			soap_docs = document_folder.get_visual_progress_notes(episodes = episodes, encounter = encounter)
+			if len(soap_docs) > 0:
+				for soap_doc in soap_docs:
+					parts = soap_doc.parts
+					if len(parts) == 0:
+						continue
+					part = parts[0]
+					fname = part.export_to_file()
+					if fname is None:
+						continue
+
+					# create bitmap
+					img = gmGuiHelpers.file2scaled_image (
+						filename = fname,
+						height = 30
+					)
+					#bmp = wx.StaticBitmap(self, -1, img, style = wx.NO_BORDER)
+					bmp = wx_genstatbmp.GenStaticBitmap(self, -1, img, style = wx.NO_BORDER)
+
+					# create tooltip
+					img = gmGuiHelpers.file2scaled_image (
+						filename = fname,
+						height = 150
+					)
+					tip = agw_stt.SuperToolTip (
+						u'',
+						bodyImage = img,
+						header = _('Created: %s') % part['date_generated'].strftime('%Y %B %d').decode(gmI18N.get_encoding()),
+						footer = gmTools.coalesce(part['doc_comment'], u'').strip()
+					)
+					tip.SetTopGradientColor('white')
+					tip.SetMiddleGradientColor('white')
+					tip.SetBottomGradientColor('white')
+					tip.SetTarget(bmp)
+
+					bmp.doc_part = part
+					bmp.Bind(wx.EVT_LEFT_UP, self._on_bitmap_leftclicked)
+					# FIXME: add context menu for Delete/Clone/Add/Configure
+					self._SZR_soap.Add(bmp, 0, wx.LEFT | wx.RIGHT | wx.TOP | wx.BOTTOM | wx.EXPAND, 3)
+					self.__bitmaps.append(bmp)
+
+		self.GetParent().Layout()
+	#--------------------------------------------------------
+	def clear(self):
+		for child_idx in range(len(self._SZR_soap.GetChildren())):
+			self._SZR_soap.Detach(child_idx)
+		for bmp in self.__bitmaps:
+			bmp.Destroy()
+		self.__bitmaps = []
+	#--------------------------------------------------------
+	def _on_bitmap_leftclicked(self, evt):
+		wx.CallAfter (
+			edit_visual_progress_note,
+			doc_part = evt.GetEventObject().doc_part,
+			discard_unmodified = True
+		)
+#============================================================
 from Gnumed.wxGladeWidgets import wxgVisualSoapPnl
 
 class cVisualSoapPnl(wxgVisualSoapPnl.wxgVisualSoapPnl):
@@ -1814,7 +2243,7 @@ class cVisualSoapPnl(wxgVisualSoapPnl.wxgVisualSoapPnl):
 
 		folder = patient.get_document_folder()
 		soap_docs = folder.get_documents (
-			doc_type = visual_progress_note_document_type,
+			doc_type = gmDocuments.DOCUMENT_TYPE_VISUAL_PROGRESS_NOTE,
 			encounter = encounter['pk_encounter']
 		)
 
@@ -2031,7 +2460,7 @@ if __name__ == '__main__':
 
 	#----------------------------------------
 	def test_select_narrative_from_episodes():
-		pat = gmPerson.ask_for_patient()
+		pat = gmPersonSearch.ask_for_patient()
 		gmPatSearchWidgets.set_active_patient(patient = pat)
 		app = wx.PyWidgetTester(size = (200, 200))
 		sels = select_narrative_from_episodes()
@@ -2040,14 +2469,14 @@ if __name__ == '__main__':
 			print sel
 	#----------------------------------------
 	def test_cSoapNoteExpandoEditAreaPnl():
-		pat = gmPerson.ask_for_patient()
+		pat = gmPersonSearch.ask_for_patient()
 		application = wx.PyWidgetTester(size=(800,500))
 		soap_input = cSoapNoteExpandoEditAreaPnl(application.frame, -1)
 		application.frame.Show(True)
 		application.MainLoop()
 	#----------------------------------------
 	def test_cSoapPluginPnl():
-		patient = gmPerson.ask_for_patient()
+		patient = gmPersonSearch.ask_for_patient()
 		if patient is None:
 			print "No patient. Exiting gracefully..."
 			return
@@ -2064,4 +2493,3 @@ if __name__ == '__main__':
 	#test_cSoapPluginPnl()
 
 #============================================================
-
